/*
 * wrpCharCount.java
 *
 * Created on December 22, 2004, 10:19 PM
 */

/**
 *
 *<p>
 * This class illustrates how to create new Java GUI applications
 * without having to write much code.
 *</p><p>
 * This class extends the class <b>wrpSampler</b>,
 * which provides the bulk of the necessary code.
 * For more information, see the <b>wrpSampler</b> class.
 * </p><p>
 * The following routines were written for this program,
 * which override the corresponding methods in the base class:
 * <dl>
 * <dt>initValues()</dt>
 * <dd>This method sets the program name and user instructions for this program</dd>
 * <dt>processData()</dt>
 * <dd>This method performs the actual operations unique to this program.</dd>
 * <dd>It is called when the user clicks on the 'Process' button</dd>
 * </dl>
 * </p><p>
 * In addition to the above methods, the new method <b>calculateLengths()</b>
 * was written to do the actual character counting.  This was done to simplify
 * the <b>processData()</b> method.  It could also allow subclass of this program
 * to implement a different counting algorithms (e.g., count words, sentences, etc.)
 * </p>
 *
 * @author  Bill Pringle
 *
 * @see wrpSampler
 * @see wrpWordCount
 * @see wrpEncrypt
 *
 */
public class wrpCharCount extends wrpSampler {
    
    /** Creates a new instance of wrpCharCount */
    public wrpCharCount() {
    }
    
    public void initValues()
    {
        programName = new String("Character Counting Program");
        helpText = new String("Enter the text to be counted in the top box and click the 'Process' button");
    }
    
    public void processData()
    {
        String txt = getTopText();
        debugPrint("processData", "Text is " + txt + ", TopText=" + topText);
        String ret = calculateLengths(txt);
        setBottomText(ret);
    }
    
    public String calculateLengths(String s)
    {
        StringBuffer sb = new StringBuffer();
        sb.append("Total characters=" + s.length() + "\n");
        // counters for various character types
        int numLetters=0, numLower=0, numUpper=0, numDigits=0, numSpace=0, numOther=0;
        
        for (int i=0; i < s.length(); i++)
        { // loop throug string and update totals
            // general characteristics are letters, digits, and other
            char c = s.charAt(i);
            if (Character.isLetter(c))
            {
                numLetters++;
                if (Character.isLowerCase(c)) numLower++;
                else numUpper++;
            }
            else if (Character.isDigit(c)) numDigits++;
            else if (Character.isSpaceChar(c)) numSpace++;
            else numOther++;
        }
        
        sb.append("Total Letters=" + numLetters);
        sb.append(" (Lower Case=" + numLower);
        sb.append(", Upper Case=" + numUpper + ")\n");
        sb.append("Total Digits=" + numDigits);
        sb.append(", Total Spaces=" + numSpace);
        sb.append(", Total Other=" + numOther + "\n");
        
        return sb.toString();
    }
    
    public static void main(String args[]) {
        new wrpCharCount().show();
    }

}
