/*
 * wrpSampler.java
 *
 * Created on December 21, 2004, 2:20 PM
 */

/**
 * <p>
 * This class is the base class for a simple application.  
 * By extending this class, you can build a GUI Java application
 *  that does what you want, writing only a minimal amount of new code.
 * </p><p>
 * The base application has two text areas: one at the top of the window for user input,
 * and one at the bottom for program output.
 * In the middle is a panel with buttons and a label containing user instructions.
 * </p><p>
 * The user should enter something in the upper text area and press the 'Process' button.
 * Once pressed, the <b>processData()</b> method is called.
 * This is where the main work is done by this program.
 * </p><p>
 * This sample program merely copies the text from the top area to the bottom area.
 * By extending this class and implementing your own <b>processData()</b> method,
 * you can create a program to do whatever you want it to do.
 * </p><p>
 * The following steps should be taken to create a new program:
 * <ol>
 * <li>Create a new class that extends wrpSampler</li>
 * <li>Implement the <b>initValues()</b> method</li>
 * <li>Implement the <b>processData()</b> method</li>
 * <li>Implement the <b>main</b> method</li>
 * <li>Implement any additional methods you need</li>
 * </ol>
 * </p><p>
 * The <b>initValues()</b> method should set two string variables:
 * <dl>
 * <dt>programName</dt>
 * <dd>The name of your program.</dd>
 * <dd>This value will be displayed at the top of the program window</dd>
 * <dt>helpString</dt>
 * <dd>User instructions</dd>
 * <dd>This will be displayed in the middle panel, under the buttons</dd>
 * </dl>
 * </p><p>
 * The <b>processData()</b> method performs the specific operations for your program.
 * It has the following class methods available:
 * <dl>
 * <dt>getTopText()</dt><dd>Returns the text that the user entered in the top area</dd>
 * <dt>setBottomText(String s)</dt><dd>Sets the lower text area to the string 's'</dd>
 * <dt>debugPrint(String r, String m)</dt>
 * <dd>Displays debugging information on the <i>System.out</i> device</dd>
 * <dd>The arguments are the routine name (r), and an optional message (m)</dd>
 * </dl>
 * </p><p>
 * The <b>main</b> method should use the <b>show()</b> method to create and start the program.
 * </p><p>
 * For example, look at the following subclasses:
 * <dl>
 * <dt>wrpCountChar</dt>
 * <dd>Counts characters by types</dd>
 * </dl>
 *
 * @author  Bill Pringle
 *
 * @see wrpCharCount
 * @see wrpWordCount
 * @see wrpEncrypt
*/

public class wrpSampler extends javax.swing.JFrame {
    /**
     * This string should be set to the program name in the initValues() method
     **/
    public String programName = new String("wrp Sample Program");
    /**
     * This string should contain user instructions.
     * It should be initialized in the initValues() method.
     */
    public String helpText = new String("Default Help Message");
    /**
     * Debugging switch.  If set to true, debug statement may be displayed
     */
    public boolean debugSwitch = true;
    /**
     * This string will be set to whatever the user entered.
     * It is initialized before calling the ProcessData() method.
     */
    public String topText;
    /**
     * This string will be set to whatever is in the lower text area.
     * It is initialized just after returning from the ProcessData() method
     */
    public String bottomText;
    
    /**
     * Creates new form wrpSampler 
     * The init() method initializes all of the necessary values
     */
    public wrpSampler() {
        debugPrint("Constructor", "wrpSampler");
        init();
    }

    /**
     * Initializes the form by calling the methods
     *  initValues(), initComponents(), and initEnvironment()
     */
    public void init()
    {
        debugPrint("init", "wrpSampler");
        initValues();
        initComponents();
        initEnvironment();
    }

    /**
     * Initializes class variables.  This method should be overridden by each subclass,
     * and set the <b>programName</b> and <b>helpText</b> strings.
     */
    public void initValues()
    {
        debugPrint("initValues", "wrpSampler");
        programName = new String("wrpSampler");
        helpText = new String("Enter text in the top panel, then click on the 'Process' button");
    }
    
    /**
     * initializes the title and help labels, 
     * using the class variables that were set in the initValues() method.
     * This method probably shouldn't be overridded by any subclass.
     */
    public void initEnvironment()
    {
        debugPrint("initEnvironment", "wrpSampler");
        lblTitle.setText(programName);
        lblHelp.setText(helpText);
    }
    
    /**
     * Can be used to set the top text area.
     */
    public void setTopText(String s)
    {
        debugPrint("setTopText", s);
        txtTop.setText(s);
    }

    /**
     * Can be used to get the string in the top text area
     */
    public String getTopText()
    {
        String s = txtTop.getText();
        debugPrint("getTopText", s);
        return s;
    }
    
    /**
     * Can be used to set the lower text area.
     */
    public void setBottomText(String s)
    {
        if (s==null) s = ""; // empty string
        debugPrint("setBottomText", s);
        txtBottom.setText(s);
    }
    
    /**
     * Can be used to set the bottom text area
     */
    public String getBottomText()
    {
        String s = txtBottom.getText();
        debugPrint("getBottomText", s);
        return s;
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        pnlTop = new java.awt.Panel();
        lblTitle = new java.awt.Label();
        jscTop = new javax.swing.JScrollPane();
        txtTop = new javax.swing.JTextArea();
        pnlMiddle = new javax.swing.JPanel();
        pnlUser = new javax.swing.JPanel();
        pnlButtons = new javax.swing.JPanel();
        btnProcess = new javax.swing.JButton();
        btnClear = new javax.swing.JButton();
        btnQuit = new javax.swing.JButton();
        pnlLabel = new javax.swing.JPanel();
        lblHelp = new javax.swing.JLabel();
        pnlBottom = new java.awt.Panel();
        jscBottom = new javax.swing.JScrollPane();
        txtBottom = new javax.swing.JTextArea();

        getContentPane().setLayout(new java.awt.GridLayout(3, 1));

        setName("frmMain");
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                exitForm(evt);
            }
        });

        pnlTop.setLayout(new java.awt.BorderLayout());

        lblTitle.setAlignment(java.awt.Label.CENTER);
        lblTitle.setFont(new java.awt.Font("Arial", 1, 14));
        pnlTop.add(lblTitle, java.awt.BorderLayout.NORTH);

        txtTop.setColumns(40);
        txtTop.setRows(5);
        jscTop.setViewportView(txtTop);

        pnlTop.add(jscTop, java.awt.BorderLayout.SOUTH);

        getContentPane().add(pnlTop);

        pnlMiddle.setLayout(new java.awt.GridLayout(3, 1));

        pnlMiddle.add(pnlUser);

        pnlButtons.setMinimumSize(new java.awt.Dimension(225, 20));
        btnProcess.setLabel("Process Data");
        btnProcess.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnProcessActionPerformed(evt);
            }
        });

        pnlButtons.add(btnProcess);

        btnClear.setText("Clear");
        btnClear.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnClearActionPerformed(evt);
            }
        });

        pnlButtons.add(btnClear);

        btnQuit.setLabel("Quit");
        btnQuit.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnQuitActionPerformed(evt);
            }
        });

        pnlButtons.add(btnQuit);

        pnlMiddle.add(pnlButtons);

        lblHelp.setFont(new java.awt.Font("MS Sans Serif", 0, 12));
        lblHelp.setHorizontalAlignment(javax.swing.SwingConstants.LEFT);
        lblHelp.setText("Enter information in the top panel, then click on the 'Process' button");
        pnlLabel.add(lblHelp);

        pnlMiddle.add(pnlLabel);

        getContentPane().add(pnlMiddle);

        txtBottom.setColumns(40);
        txtBottom.setRows(5);
        jscBottom.setViewportView(txtBottom);

        pnlBottom.add(jscBottom);

        getContentPane().add(pnlBottom);

        pack();
    }//GEN-END:initComponents

    /**
     * Handler for the 'Clear' button
     */
    private void btnClearActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnClearActionPerformed
        // TODO add your handling code here:
        clearDisplay();
    }//GEN-LAST:event_btnClearActionPerformed

    /**
     * Should clear the upper and lower text areas.
     * Probably should not be overridden in subclasses,
     * unless you want to implement an undo feature.
     */
    public void clearDisplay()
    {
        if (debugSwitch) debugPrint("clearDisplay");
        setTopText("");
        setBottomText("");
    }
    
    /**
     * Handler for the 'Quit' button
     */
    private void btnQuitActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnQuitActionPerformed
        // TODO add your handling code here:
        quitProgram();
    }//GEN-LAST:event_btnQuitActionPerformed

    public void quitProgram()
    {
        debugPrint("quitProgram", "wrpSampler");
        System.exit(0);
    }
    
    /**
     * Handler for the 'Process' button.
     * This method will set the string <b>topText</b> and <b>bottomText</b>
     * before and after calling <b>ProcessData</b>, respectively.
     */
    private void btnProcessActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnProcessActionPerformed
        // TODO add your handling code here:
        topText = getTopText(); // get the text in the top area
        processData(); // call the specific process routine for this program
        bottomText = getBottomText(); // get the text that was placed in the bottom area
    }//GEN-LAST:event_btnProcessActionPerformed

    /**
     * The method that should implement the main processing actions of the program.
     * This method should be overridden by all subclasses.
     * It can reference the string <b>topText</b> to get what the user entered in the top area.
     * Once done, the resulting text in the lower area will be saved in the string <b>bottomText</b>
     */
    public void processData()
    {
        String txt = getTopText();
        setBottomText(txt);
    }
    
    /** Exit the Application */
    private void exitForm(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_exitForm
        quitProgram();
    }//GEN-LAST:event_exitForm
    
    /**
     * Displays debugging information.
     *
     * @param rtn Name of the routine where the statement is located
     * @param s Information string to be displayed.
     */
    public void debugPrint(String rtn, String s)
    {
        System.out.println("Program: " + programName + ", method: " + rtn + ", msg: " + s + ".");
    }
    
    /**
     * Display debugging information, but without a specific message.
     *
     * @param rtn Name of the routine
     */
    public void debugPrint(String rtn)
    {
        debugPrint(rtn, "[no message]");
    }
    /**
     * @param args the command line arguments
     */
    public static void main(String args[]) {
        new wrpSampler().show();
    }
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton btnClear;
    private javax.swing.JButton btnProcess;
    private javax.swing.JButton btnQuit;
    private javax.swing.JScrollPane jscBottom;
    private javax.swing.JScrollPane jscTop;
    private javax.swing.JLabel lblHelp;
    private java.awt.Label lblTitle;
    protected java.awt.Panel pnlBottom;
    protected javax.swing.JPanel pnlButtons;
    private javax.swing.JPanel pnlLabel;
    protected javax.swing.JPanel pnlMiddle;
    protected java.awt.Panel pnlTop;
    public javax.swing.JPanel pnlUser;
    private javax.swing.JTextArea txtBottom;
    protected javax.swing.JTextArea txtTop;
    // End of variables declaration//GEN-END:variables
    
}
